<?php
/**
 * Import_HTML_Form
 * 
 * @package 
 * @author Gabriel Comarita
 * @copyright 2012
 * @version $Id$
 * @access public
 */
class Import_HTML_Form extends Manage_Forms_Fields
{
    var $form_elements = 'input, select, textarea, button'; // default form elements to search
	
    var $skip_element_name = array('form_id'); // Skip element of name in array
    var $skip_element_type = array('submit', 'image', 'file'); //skip element of type in array
    
    var $input_text_aliases = array('text', 'tel', 'email', 'url');
	
    var $replace_with_previous_label = false;
    var $get_previous_text_for_option = false;
    
    var $html_dom = NULL;
    var $form = NULL;
    var $element_name = NULL;
    var $element_name_no_bracket = NULL;// no array looks ver of $element_name
    var $element_tag = NULL;
    var $element_type = NULL;
    var $child = NULL;	
    var $output = array();
    
    var $conf;
    var $db;
	
	/**
	 * Contructor
	 * Parse html, process to get the desired output
	 *
	 * @access public
	 * @param string $form 
	 * @param string $form_elements 
	 * @return void
	 */
	function __construct($form = NULL, $form_elements = NULL, $conf, $db)
	{
            $this->conf = $conf;
            $this->db = $db;	   

            // Create a dom object
            $this->html_dom = afp_str_get_html($form);

            // Find form
            $this->form = $this->html_dom->find('form');

            if($form_elements) {
                $this->form_elements = $form_elements;	
            }	

            $this->process();
            $this->filter_element();
	} // End contructor
	
	/**
	 * Main Processing
	 *
	 * @access private
	 * @return void
	 */
	private function process()
	{
		foreach($this->form as $element)
		{
			foreach($this->html_dom->find($this->form_elements) as $child)	
			{
				$this->child = $child;
				$this->element_name = NULL; // Nullify previous element name
				$this->get_element_primary_info();	
				if($this->element_name)
				{
				    $this->parseElementType($this->element_type);
					$this->get_element_details();					
				}
			}	
		}		
	}//end function process
	
	/**
	 * Get Primary info - name, attributes
	 *
	 * @access private
	 * @return void
	 */
	private function get_element_primary_info()
	{
		if($this->child)	
		{
			// Only having 'name' attribute
			if(isset($this->child->attr["name"]) && $this->child->attr["name"])
			{
				$this->element_name = $this->clean_space($this->child->attr["name"]);
				$this->element_name_no_bracket = $this->array_key_to_string($this->element_name);
			}
			
			$this->element_tag = 'input'; // Default element tag
			
			if(isset($this->child->tag) && $this->child->tag)
			{
				$this->element_tag = $this->clean_space($this->child->tag);
			}
			
			// Search with element name and get element type
			if($this->element_name)
			{
				foreach($this->html_dom->find($this->element_tag.'[name='.$this->element_name.']') as $element)
				{
					if(isset($element->attr["type"]) && $element->attr["type"])
					{
						$this->element_type = $this->clean_space($element->attr["type"]);
					}
					else if(isset($this->child->tag) && $this->child->tag)
					{
						$this->element_type = $this->clean_space($this->child->tag);
					}
				}	
			}
		}
	}// End function get_element_primary_info
	
	/**
	 * Get element details
	 * Skip element if element name is in 'skip_element_name'
	 * Skip element if element type is in 'skip_element_name'
	 *
	 * @access private
	 * @return void
	 */
	private function get_element_details()
	{
		if(!in_array($this->clean_space(strtolower($this->element_name_no_bracket)), $this->skip_element_name) && 
		!in_array($this->clean_space(strtolower($this->element_type)), $this->skip_element_type))
		{
			$this->output[$this->element_name_no_bracket] = $this->clean_space($this->child->attr);			
			
			if(!isset($this->output[$this->element_name_no_bracket]["type"]))	
			{
				$this->output[$this->element_name_no_bracket]["type"] = $this->clean_space($this->element_tag);
			}
			
			/**
			 * Process for input type radio and checkbox
			 * Get values of elements in array
			 * Remove attributes - class, id, value from output
			 */
			if($this->element_type == 'radio' || $this->element_type == 'checkbox')
			{
				$this->output[$this->element_name_no_bracket]["options"] = $this->find_input_option();
				
				if(isset($this->output[$this->element_name_no_bracket]["value"]))
				{
					unset($this->output[$this->element_name_no_bracket]["value"]);
				}
				if(isset($this->output[$this->element_name_no_bracket]["class"]))
				{
					unset($this->output[$this->element_name_no_bracket]["class"]);
				}
			}
            
            /*
            // Remove ID attribute for any type of element
			if(isset($this->output[$this->element_name_no_bracket]["id"]))
			{
				unset($this->output[$this->element_name_no_bracket]["id"]);
			}
            */
                			
			/**
			 * Process for element select
			 * Get options' value/text
			 */
			if($this->element_type == 'select')
			{
				$this->output[$this->element_name_no_bracket]["options"] = $this->find_select_option();			
			}
			
			if(!isset($this->output[$this->element_name_no_bracket]["value"]))
			{
				foreach($this->html_dom->find($this->element_tag.'[name='.$this->element_name.']') as $element) 
				{
					if(isset($element->innertext) && $element->innertext && $this->element_tag != 'select')	
					{
						$this->output[$this->element_name_no_bracket]["value"] = $this->clean_space($element->innertext);	
					}
				}
			}
			
			/**
			 * Replace the name key from ouput with previous sibling (if only label)
			 * Applicable for element type - text, select, textarea
			 */
			if($this->element_type == 'select' ||
			$this->element_type == 'text' ||
			$this->element_type == 'textarea')
			{
				$key_name = '';
				
                if($this->replace_with_previous_label) {
                
    				foreach($this->html_dom->find($this->element_tag.'[name='.$this->element_name.']') as $element)
    				{
    					if($prev_element = $element->prev_sibling())
    					{
    						foreach($prev_element->find('label') as $pe)
    						{
    							$key_name = $pe->plaintext;		
                            }
    					}	
    					else if($element->parent()->tag == 'label')
    					{
    						$key_name = $e->parent()->plaintext;	
    					}
    					else if($element->parent()->tag != 'label')
    					{
    						foreach($element->parent()->find('label') as $pnl)
    						{
    							$key_name = $pnl->plaintext;		
    						}
    	
    						if(!$key_name)
    						{
    							$prev_parent_sibling = $element->parent()->prev_sibling();
    							if($prev_parent_sibling->tag == 'label')
    							{
    								$key_name = $prev_parent_sibling->plaintext;
    							}	
    						}
    					}
    				}
                    
                }
				
				if($key_name)
				{
					$this->output[$this->clean_space($key_name)] = $this->output[$this->element_name_no_bracket];	
					unset($this->output[$this->element_name_no_bracket]);
				}
			}
			
		}
	}//end function get_element_details
	
	/**
	 * Get options for select input type
	 *
	 * @access private
	 * @return array
	 */
	private function find_select_option()
	{
            $options = array();

            foreach($this->html_dom->find('select[name='.$this->element_name.']') as $element) 
            {
                foreach($element->find('option') as $element) {
                    if(isset($element->value) && $element->value)
                    {
                        $element_value = $this->strip_quotes($element->value);
                                                
                        $options[$this->clean_space($element_value)] = $this->clean_space($element->innertext);
                    } else {
                        $options[$this->clean_space($element->innertext)] = $this->clean_space($element->innertext);
                    }
                }	
            }
		return $options;
	} // End function find_select_option
	
	/**
	 * Get values of element in array
	 * Input tag - input
	 *
	 * @access private
	 * @return array
	 */
	private function find_input_option()
	{
		$options = array();
		
		foreach($this->html_dom->find('input[name^='.$this->element_name_no_bracket.']') as $element) 
		{
			if($this->element_name_no_bracket == $this->array_key_to_string($element->attr["name"]))
			{
				if($element->attr["value"])
				{   
				    // Get the text
				    if($this->get_previous_text_for_option) {
				        
    					$parent_or_sibling = '';
    					
    					foreach($this->html_dom->find('input[value='.$element->attr["value"].']') as $e) 
    					{
    						if($e->parent()->tag == 'label')
    						{
    							$parent_or_sibling = $e->parent()->plaintext;	
    						}
    						else if($e->parent()->tag != 'label')
    						{
    							foreach($e->parent()->find('label,a') as $pnl)
    							{
    								$parent_or_sibling = $pnl->plaintext;		
    							}
    						}
    						// Sibling element comes before element
    						else if($prev_element = $e->prev_sibling())
    						{
    							foreach($prev_element->find('label') as $pe)
    							{
    								$parent_or_sibling = $pe->plaintext;		
    							}							
    						}		
    					}		
    					
    					// Check for innertext of parent/sibling element
    					if($parent_or_sibling)		
    					{
    						$options[$this->clean_space($element->attr["value"])] = $this->clean_space($parent_or_sibling);		
    					}
    					else if($text = $e->find('text', 0))
    					{
    						$options[$this->clean_space($element->attr["value"])] = $text->plaintext;	
    					}
    					else
    					{
    						$options[] = $this->clean_space($element->attr["value"]);	
    					}
                    
                    // Get the Value
                    } else {
                        $value_str = $this->clean_space($element->attr["value"]);
                        
                        $options[$value_str] = $value_str;
                    }
				}	
			}
		}
		return $options;
	}// End function find_input_option
	
	/**
	 * Filter output element(s)
	 * Remove 'id' key from the output
	 *
	 * @access private
	 * @return void
	 */
	private function filter_element()
	{
		if($this->output && is_array($this->output))
		{
			foreach($this->output as $key => $value)
			{
				if(isset($value["id"]))
				{
					unset($this->output[$key]["id"]);	
				}	
			}
		}
	} // End function filter_element
	
	/**
	 * Customize string
	 * Convert name/name[]/name[anyname] to name
	 *
	 * @access private
	 * @param string $param
	 * @return string
	 */
	private function array_key_to_string($param = NULL)
	{
		if($param)
		{
			$param = $this->clean_space($param);
			$param = str_replace("[","*_*", $param);
			$param = str_replace("]","", $param);
			$param = explode("*_*", $param);			
			$param = $param[0];			
		}
		return $param;
	} // End function array_key_to_string
	
	/**
	 * Preint/Return Output
	 * 
	 * @access public
	 * @param boolean $print
	 * @return mixed 
	 */
	public function render_output($print = true)
	{
		if($this->output)
		{
			if($print)
			{
				echo '<pre>'; print_r($this->output); echo '</pre>';
			}
			else
			{
				return $this->output;
			}			
		}
	} // End function render_output
	
	/**
	 * Flush class variables and ouput for next process
	 * 
	 * @access public
	 * @return void
	 */
	public function clear()
	{
		$this->html_dom = NULL;
		$this->form = NULL;
		$this->element_name = NULL;
		$this->element_name_no_bracket = NULL;
		$this->element_tag = NULL;
		$this->element_type = NULL;
		$this->child = NULL;	
		$this->output = array();	
	} // End function clear
	
	/**
	 * Trim spaces
	 *
	 * @access private
	 * @param mixed $param
	 * @return mixed
	 */
	private function clean_space($param)
	{
		if(is_array($param))
		{
			return array_filter(array_map('trim', $param));
		}
		if(is_string($param))
		{
			return trim($param);	
		}
	} // End function clean_space
    
    private function strip_quotes($param) {
        return trim(str_replace(array('\"',"\'"), '', $param));
    }    
        
    public function import2DB($form_id)
    {
        $imported_form_elements = $this->render_output(0); #exit;
        
        $this->clear();
        
        if( ! empty($imported_form_elements) ) {
            
            foreach($imported_form_elements as $field_title => $field_properties) {
                
                $field_title = str_replace('_', ' ', $field_title);
                $field_title = ucwords($field_title);
                
                $field_name = $field_properties['name'];
                $field_name = str_replace('[]', '', $field_name);
                
                $field_type = $field_properties['type'];
                         
                $field_type_id = $this->getFieldTypeID($field_type);
                
                if(!$field_type_id) {
                    continue;
                }
                
                $columns = (in_array($field_type, array('radio', 'checkbox')));
    
                $data = array(
                    'form_id'    => $form_id,
                    'text'       => $field_title,
                    'name'       => $field_name,
                    'mandatory'  => 0,
                    'columns'    => $columns,
                    'type_id'    => $field_type_id,
                    'position'   => 0,
                    'parent_id'  => 0
                );
                               
                $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields', $data));
                
                $field_id = $this->db->insertId();
                
                /*
                -----------
                TEXT Field
                -----------
                */
                if($field_type == 'text') {
                    
                    if( (count($field_properties) - 1) > 0 ) { // Exclude the "type" attribute
                        
                        foreach($field_properties as $attribute_name => $attribute_value) {
                            
                            if($attribute_name == 'type'
                            || $attribute_value == 'name'
                            || $attribute_value == '') {
                                continue;
                            }
                            
                            $attributes_data = array(
                                'field_id' => $field_id,
                                'name' => $attribute_name,
                                'value' => $attribute_value
                            );
                            
                            $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields_attributes', $attributes_data));
                        }
                    }
                    
                /*
                ----------------
                SELECT drop-down
                ----------------
                */
                } else if($field_type == 'select') {
                    
                    if( (count($field_properties) - 2) > 0 ) { // Exclude the "type" attribute and the "options" key
                        foreach($field_properties as $attribute_name => $attribute_value) {

                            if($attribute_name == 'type'
                            || $attribute_name == 'options'
                            || $attribute_name == 'size'
                            || $attribute_name == 'name'
                            || $attribute_name == 'id'
                            || $attribute_value == '') {
                                continue;
                            }

                            $attributes_data = array(
                                'field_id' => $field_id,
                                'name' => $attribute_name,
                                'value' => $attribute_value
                            );
                            
                            $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields_attributes', $attributes_data));
                        }                       
                    }
                    
                    $o_position = 1;
                    
                    foreach($field_properties['options'] as $o_value => $o_text) {
                        
                        if($o_value == '' || $o_text == '') {
                            continue;    
                        }
                        
                        $options_data = array(
                            'field_id'   => $field_id,
                            'text'       => $o_text,
                            'value'      => (($o_value) ? $o_value : $this->generateFieldName($o_text)),
                            'attributes' => '',
                            'position'   => $o_position,
                            'parent_id'  => 0
                        );
                        
                        $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields_options', $options_data));
                        
                        $o_position++;
                    }                    

                /*
                ------------------------
                Checkbox and Radio Type
                ------------------------
                */
                } else if($field_type == 'checkbox' || $field_type == 'radio') {
                    $o_position = 1;
                    
                    foreach($field_properties['options'] as $o_value => $o_text) {
                        if($o_value == '' && $o_value != '0') continue; // Skip options without any values
                        
                        $options_data = array(
                            'field_id'   => $field_id,
                            'text'       => $o_text,
                            'value'      => $o_value,
                            'attributes' => '',
                            'position'   => $o_position,
                            'parent_id'  => 0
                        );
                        
                        $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields_options', $options_data));
                        
                        $o_position++;
                    }
                /*
                ------------
                Hidden Type
                ------------
                */
                } else if($field_type == 'hidden') {

                    $attributes_data = array(
                        'field_id' => $field_id,
                        'name' => 'value',
                        'value' => $field_properties['value']
                    );
                    
                    $this->db->query($this->db->prepareInsert($this->conf['db']['prefix'].'fields_attributes', $attributes_data));
                }    
            }
        }
        return true;        
    }
    
    /**
     * Import_HTML_Form::getFieldTypeID()
     * 
     * @param mixed $field_type
     * @return
     */
    public function getFieldTypeID($field_type) {
           
        if(in_array(strtolower($field_type), $this->input_text_aliases)) {
            $field_type = 'input';
        }
        
        foreach($this->getTypes() as $a_key => $a_type) {
            if(preg_match('/'.$field_type.'/i', $a_type)) {
                return $a_key;
            }
        }
    }
    
    /**
     * Import_HTML_Form::parseElementType()
     * 
     * @param mixed $type
     * @return void
     */
    private function parseElementType($type) {
        if(in_array(strtolower($type), $this->input_text_aliases)) {
            $this->element_type = 'text';
        } else {
            $this->element_type = $type;
        }
    }
} // End class Import_HTML_Form
?>