<?php
/**
 * Validation
 * 
 * @package AJAX Form Pro
 * @author Gabriel Comarita
 * @copyright 2011
 * @version $Id$
 * @access public
 */
class Validation {
    
    var $conf;
    var $db;
    var $afb_form_fields;
    
    /**
     * Validation::__construct()
     * 
     * @param mixed $conf
     * @param mixed $db
     * @return void
     */
    function __construct($conf = array(), $db = array()) {
        $this->conf = $conf;
        $this->db = $db;        
    }
    
    /**
     * Validation::DoValidate()
     * 
     * @param mixed $value
     * @param mixed $validation
     * @return
     */
    function DoValidate($value, $validation, $field_id = 0) {
                
        if(array_key_exists('basic', $validation) || array_key_exists('radio', $validation)) { 
            if( ! $this->ValidateBasic($value)) {
                return 'basic';
            }
	}
        
        if(array_key_exists('email', $validation)) { 
            if( ! $this->ValidateEmail($value)) {
                return 'email';
            }
	}
        
        if(array_key_exists('phone', $validation)) { 
            if( ! $this->ValidatePhoneNumber($value, $validation['phone']['value'])) {
                return 'phone';
            }
	}        

        if(array_key_exists('numeric', $validation)) { 
            if( ! $this->ValidateNumber($value)) {
                return 'numeric';
            }
	}

        if(array_key_exists('min_selections', $validation)) {             
            if( ! $this->ValidateMinSelections($value, $validation['min_selections']['value'])) {
                return 'min_selections';
            }
	}
        
        if(array_key_exists('min_chars', $validation)) { 
            if( ! $this->ValidateMinChars($value, $validation['min_chars']['value'])) {
                return 'min_chars';
            }
	}
        
        if(array_key_exists('max_chars', $validation)) { 
            if( ! $this->ValidateMaxChars($value, $validation['max_chars']['value'])) {
                return 'max_chars';
            }
        }

        if(array_key_exists('eq_chars', $validation)) { 
            if( ! $this->ValidateEqChars($value, $validation['eq_chars']['value'])) {
                return 'eq_chars';
            }
        }
        
        if(array_key_exists('f1_eq_f2', $validation)) {
            if( ! $this->ValidateF1EqF2($value, $validation['f1_eq_f2']['value']) || !$value ) {
                return 'f1_eq_f2';
            }            
        }
        
        if(array_key_exists('regexp', $validation)) { 
            if( ! $this->ValidateRegExp($value, $validation['regexp']['value'])) {
                return 'regexp';
            }
        }        
        
        if($field_id) {
            if(array_key_exists('no_dupes', $validation)) { 
                if( ! $this->ValidateNoDupes($value, $field_id)) {
                    return 'no_dupes';
                }
    	    }
        }
        
        return true;
    }

    /**
     * Validation::ValidateBasic()
     * 
     * @param mixed $value
     * @return
     */
    function ValidateBasic($value) {
        if($value == '' || empty($value)) {
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateEmail()
     * 
     * @param mixed $value
     * @return
     */
    function ValidateEmail($value)
    {
        return filter_var($value, FILTER_VALIDATE_EMAIL);
        }

    /**
     * Validation::ValidatePhoneNumber()
     * 
     * @param mixed $number
     * @param mixed $formats
     * @return
     */
    function ValidatePhoneNumber($number, $formats) {
        $formats_list = array_map('trim', explode(',', $formats));
        
        $number = str_replace('#', '', $number);
        $format = trim(preg_replace('/[0-9]/', '#', $number));
        
        return (in_array($format, $formats_list)) ? true : false;       
    }
    
    /**
     * Validation::ValidateMinSelections()
     * 
     * @param mixed $value
     * @param string $min_selections
     * @return
     */
    function ValidateMinSelections($value, $min_selections = '') {        
        if($min_selections > count($value)) {
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateMinChars()
     * 
     * @param mixed $value
     * @param string $min_chars
     * @return
     */
    function ValidateMinChars($value, $min_chars = '') {
        if(strlen($value) < $min_chars) { 
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateMaxChars()
     * 
     * @param mixed $value
     * @param string $max_chars
     * @return
     */
    function ValidateMaxChars($value, $max_chars = '') {
        if(strlen($value) > $max_chars) { 
            return false;
        }
        return true;
    }

    /**
     * Validation::ValidateEqChars()
     * 
     * @param mixed $value
     * @param string $eq_chars
     * @return
     */
    function ValidateEqChars($value, $eq_chars = '') {
        if(strlen($value) != $eq_chars) { 
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateF1EqF2()
     * 
     * @param mixed $f2_value
     * @param mixed $f1_id
     * @return
     */    
    function ValidateF1EqF2($f2_value, $f1_id) {
               
        # Determine value of f1 based on ID
        foreach($this->afb_form_fields as $a_val) {
            if($a_val['field_id'] == $f1_id) {
                $f1_value = $_POST[$a_val['name']];
            }
        }
        
        if($f1_value != $f2_value) {
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateNumber()
     * 
     * @param mixed $value
     * @return
     */
    function ValidateNumber($value) {
        if( ! is_numeric($value) ) {
            return false;
        }
        return true;
    }
    
    /**
     * Validation::ValidateRegExp()
     * 
     * @param mixed $value
     * @param mixed $regexp
     * @return
     */
    function ValidateRegExp($value, $regexp) {
        if($value == '') { 
	        return false;
        } else {
            return preg_match($regexp, $value);
        }
    }
    
    /**
     * Validation::ValidateNoDupes()
     * 
     * @param mixed $value
     * @param mixed $field_id
     * @return
     */
    function ValidateNoDupes($value, $field_id) {
        $db_check = $this->db->getOne("SELECT id FROM `".$this->conf['db']['prefix']."data_fields` WHERE value='".$value."' && field_id='".$field_id."'");
        
        if($db_check) {
            return false;
}
        
        return true;
    }
}
?>